@@ -24,41 +24,16 @@
 #include <IOKit/assert.h>
 #include <IOKit/IOLib.h>
 #include <IOKit/IOMessage.h>
-#include <IOKit/IOBufferMemoryDescriptor.h>
+#include <IOKit/IOMemoryDescriptor.h>
 
 #include <IOKit/usb/IOUSBDevice.h>
 #include <IOKit/usb/IOUSBInterface.h>
 #include <IOKit/usb/IOUSBPipe.h>
 #include <IOKit/usb/IOUSBLog.h>
+#include <IOKit/IOCommandPool.h>
 
 #include "IOUSBInterfaceUserClient.h"
 
-struct AsyncPB {
-    OSAsyncReference 		fAsyncRef;
-    UInt32 			fMax;
-    IOMemoryDescriptor 		*fMem;
-    IOUSBDevRequestDesc		req;
-};
-
-struct IsoAsyncPB {
-    OSAsyncReference 	fAsyncRef;
-    int			frameLen;	// In bytes
-    void *		frameBase;	// In user task
-    IOMemoryDescriptor *dataMem;
-    IOMemoryDescriptor *countMem;
-    IOUSBIsocFrame	frames[0];
-};
-
-struct LowLatencyIsoAsyncPB {
-    OSAsyncReference 		fAsyncRef;
-    int				frameLen;	// In bytes
-    void *			frameBase;	// In user task
-    IOMemoryDescriptor *	dataMem;
-    IOMemoryDescriptor *	countMem;
-    IOUSBLowLatencyIsocFrame	frames[0];
-};
-
-
 //=============================================================================================
 //
 //	Note on the use of IncrementOutstandingIO(), DecrementOutstandingIO() and doing extra
@@ -88,6 +63,8 @@ struct LowLatencyIsoAsyncPB {
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 #define super IOUserClient
 OSDefineMetaClassAndStructors(IOUSBInterfaceUserClient, IOUserClient)
+OSDefineMetaClassAndStructors(IOUSBLowLatencyCommand, IOCommand)
+
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 enum {
@@ -258,6 +235,20 @@ IOUSBInterfaceUserClient::sMethods[kNumUSBInterfaceMethods] = {
 	3,
 	3
     },
+    { //    kUSBInterfaceUserClientLowLatencyPrepareBuffer
+	(IOService*)kMethodObjectThis,
+	(IOMethod) &IOUSBInterfaceUserClient::LowLatencyPrepareBuffer,
+	kIOUCStructIStructO,
+	sizeof(LowLatencyUserBufferInfo),
+	0
+    },
+    { //    kUSBInterfaceUserClientLowLatencyReleaseBuffer
+	(IOService*)kMethodObjectThis,
+	(IOMethod) &IOUSBInterfaceUserClient::LowLatencyReleaseBuffer,
+	kIOUCStructIStructO,
+	sizeof(LowLatencyUserBufferInfo),
+	0
+    }
 };
 
 
@@ -401,8 +392,6 @@ IOUSBInterfaceUserClient::initWithTask(task_t owningTask, void *security_id , UI
 bool 
 IOUSBInterfaceUserClient::start( IOService * provider )
 {
-    IOWorkLoop		*wl;
-    
     USBLog(7, "+%s[%p]::start(%p)", getName(), this, provider);
     
     IncrementOutstandingIO();		// make sure we don't close until start is done
@@ -411,13 +400,13 @@ IOUSBInterfaceUserClient::start( IOService * provider )
 
     if (!fOwner)
     {
- 	USBError(1, "%s[%p]::start - provider is NULL!", getName(), this);
-	goto ErrorExit;
+        USBError(1, "%s[%p]::start - provider is NULL!", getName(), this);
+        goto ErrorExit;
     }
     
     if(!super::start(provider))
     {
- 	USBError(1, "%s[%p]::start - super::start returned false!", getName(), this);
+        USBError(1, "%s[%p]::start - super::start returned false!", getName(), this);
         goto ErrorExit;
     }
 
@@ -425,26 +414,38 @@ IOUSBInterfaceUserClient::start( IOService * provider )
 
     if(!fGate)
     {
-	USBError(1, "%s[%p]::start - unable to create command gate", getName(), this);
-	goto ErrorExit;
+        USBError(1, "%s[%p]::start - unable to create command gate", getName(), this);
+        goto ErrorExit;
     }
 
-    wl = getWorkLoop();
-    if (!wl)
+    fWorkLoop = getWorkLoop();
+    if (!fWorkLoop)
     {
-	USBError(1, "%s[%p]::start - unable to find my workloop", getName(), this);
-	fGate->release();
-	goto ErrorExit;
+        USBError(1, "%s[%p]::start - unable to find my workloop", getName(), this);
+        goto ErrorExit;
     }
+    fWorkLoop->retain();
     
-    if (wl->addEventSource(fGate) != kIOReturnSuccess)
+    if (fWorkLoop->addEventSource(fGate) != kIOReturnSuccess)
     {
-	USBError(1, "%s[%p]::start - unable to add gate to work loop", getName(), this);
-	fGate->release();
-	goto ErrorExit;
+        USBError(1, "%s[%p]::start - unable to add gate to work loop", getName(), this);
+        goto ErrorExit;
+    }
+
+    fFreeUSBLowLatencyCommandPool = IOCommandPool::withWorkLoop(fWorkLoop);
+    if (!fFreeUSBLowLatencyCommandPool)
+    {
+        USBError(1,"%s[%p]::start - unable to create free command pool", getName(), this);
+        
+        // Remove the event source we added above
+        //
+        fWorkLoop->removeEventSource(fGate);
+        
+        goto ErrorExit;
     }
 
     DecrementOutstandingIO();
+    USBLog(7, "-%s[%p]::start(%p)", getName(), this, provider);
     return true;
     
 ErrorExit:
@@ -455,6 +456,12 @@ ErrorExit:
         fGate = NULL;
     }
         
+    if ( fWorkLoop != NULL )
+    {
+        fWorkLoop->release();
+        fWorkLoop = NULL;
+    }
+        
     DecrementOutstandingIO();
     return false;
 }
@@ -526,17 +533,24 @@ IOUSBInterfaceUserClient::close()
 IOReturn 
 IOUSBInterfaceUserClient::clientClose( void )
 {
-    USBLog(7, "+%s[%p]::clientClose()", getName(), this);
+    USBLog(7, "+%s[%p]::clientClose(%p)", getName(), this, fUserClientBufferInfoListHead);
 
     // Sleep for 1 ms to allow other threads that are pending to run
     //
     IOSleep(1);
     
+    // If we have any kernelDataBuffer pointers, then release them now
+    //
+    if (fUserClientBufferInfoListHead != NULL)
+    {
+        ReleasePreparedDescriptors();
+    }
+
     fTask = NULL;
     
     terminate();
 
-    USBLog(7, "-%s[%p]::clientClose()", getName(), this);
+    USBLog(7, "-%s[%p]::clientClose(%p)", getName(), this, fUserClientBufferInfoListHead);
 
     return kIOReturnSuccess;			// DONT call super::clientClose, which just returns notSupported
 }
@@ -547,12 +561,12 @@ IOUSBInterfaceUserClient::clientDied( void )
 {
     IOReturn ret;
 
-    USBLog(3, "+%s[%p]::clientDied()", getName(), this);
+    USBLog(5, "+%s[%p]::clientDied()", getName(), this);
 
     fDead = true;				// don't send any mach messages in this case
     ret = super::clientDied();
 
-    USBLog(3, "-%s[%p]::clientDied()", getName(), this);
+    USBLog(5, "-%s[%p]::clientDied()", getName(), this);
 
     return ret;
 }
@@ -623,28 +637,36 @@ void
 IOUSBInterfaceUserClient::LowLatencyIsoReqComplete(void *obj, void *param, IOReturn res, IOUSBLowLatencyIsocFrame *pFrames)
 {
     void *			args[1];
-    LowLatencyIsoAsyncPB * 	pb = (LowLatencyIsoAsyncPB *) param;
+    IOUSBLowLatencyCommand *	command = (IOUSBLowLatencyCommand *) param;
+    IOMemoryDescriptor *	dataBufferDescriptor;
+    OSAsyncReference		asyncRef;
+    
     IOUSBInterfaceUserClient *	me = OSDynamicCast(IOUSBInterfaceUserClient, (OSObject*)obj);
 
     if (!me)
 	return;
 	
-    args[0] = pb->frameBase;
-    pb->countMem->writeBytes(0, pb->frames, pb->frameLen);
-    pb->dataMem->complete();
-    pb->dataMem->release();
-    pb->countMem->release();
+    args[0] = command->GetFrameBase(); 
+    
+    command->GetAsyncReference(&asyncRef);
+    
     if (!me->fDead)
-	sendAsyncResult(pb->fAsyncRef, res, args, 1);
+	sendAsyncResult( asyncRef, res, args, 1);
+
+    // Complete the memory descriptor
+    //
+    dataBufferDescriptor = command->GetDataBuffer();
+    dataBufferDescriptor->complete();
+    dataBufferDescriptor->release();
+    
+    // Free/give back the command 
+    me->fFreeUSBLowLatencyCommandPool->returnCommand(command);
 
-    IOFree(pb, sizeof(*pb)+pb->frameLen);
     me->DecrementOutstandingIO();
     me->release(); 
 }
 
 
-
-
 IOReturn 
 IOUSBInterfaceUserClient::SetAlternateInterface(UInt8 altSetting)
 {
@@ -1423,6 +1445,317 @@ IOUSBInterfaceUserClient::ControlRequestInOOL(IOUSBDevReqOOLTO *reqIn, UInt32 *s
 }
 
 
+IOReturn 
+IOUSBInterfaceUserClient::LowLatencyPrepareBuffer(LowLatencyUserBufferInfo *bufferData)
+{
+    IOReturn				ret 			= kIOReturnSuccess;
+    IOMemoryDescriptor *		aDescriptor		= NULL;
+    LowLatencyUserClientBufferInfo *	kernelDataBuffer	= NULL;
+    IOMemoryMap *			frameListMap		= NULL;
+    IODirection				direction;
+    
+    IncrementOutstandingIO();
+    
+    if (fOwner && !isInactive())
+    {
+        USBLog(3, "%s[%p]::LowLatencyPrepareBuffer cookie: %d, buffer: %p, size: %d, type %d, isPrepared: %d, next: %p", getName(), this,
+            bufferData->cookie,
+            bufferData->bufferAddress,
+            bufferData->bufferSize,
+            bufferData->bufferType,
+            bufferData->isPrepared,
+            bufferData->nextBuffer);
+            
+        // Allocate a buffer and zero it
+        //
+        kernelDataBuffer = ( LowLatencyUserClientBufferInfo *) IOMalloc( sizeof(LowLatencyUserClientBufferInfo) );
+        if (kernelDataBuffer == NULL )
+        {
+            USBLog(1,"%s[%p]::LowLatencyPrepareBuffer  Could not malloc buffer info (size = %d)!", getName(), this, sizeof(LowLatencyUserClientBufferInfo) );
+            return kIOReturnNoMemory;
+        }
+        
+        bzero(kernelDataBuffer, sizeof(LowLatencyUserClientBufferInfo));
+        
+        // Cool, we have a good buffer, add it to our list
+        //
+        AddDataBufferToList( kernelDataBuffer );
+
+        // Set the known fields
+        //
+        kernelDataBuffer->cookie = bufferData->cookie;
+        kernelDataBuffer->bufferType = bufferData->bufferType;
+        
+        // Create a memory descriptor for our data buffer and prepare it (pages it in if necesary and prepares it)
+        //
+        if ( (bufferData->bufferType == kUSBLowLatencyWriteBuffer) || ( bufferData->bufferType == kUSBLowLatencyReadBuffer) )
+        {
+            // We have a data buffer, so create a IOMD and prepare it
+            //
+            direction = ( bufferData->bufferType == kUSBLowLatencyWriteBuffer ? kIODirectionOut : kIODirectionIn );
+            aDescriptor = IOMemoryDescriptor::withAddress((vm_address_t)bufferData->bufferAddress, bufferData->bufferSize, direction, fTask);
+            if(!aDescriptor) 
+            {
+                USBLog(1,"%s[%p]::LowLatencyPrepareBuffer  Could not create a data buffer memory descriptor (addr: %p, size %d)!", getName(), this, bufferData->bufferAddress, bufferData->bufferSize );
+                ret = kIOReturnNoMemory;
+                goto ErrorExit;
+            }
+
+            ret = aDescriptor->prepare();
+            if (ret != kIOReturnSuccess)
+            {
+                USBLog(1,"%s[%p]::LowLatencyPrepareBuffer  Could not prepare the data buffer memory descriptor!", getName(), this );
+                goto ErrorExit;
+            }
+
+            
+            // OK, now save this in our user client structure
+            // 
+            kernelDataBuffer->bufferAddress = bufferData->bufferAddress;
+            kernelDataBuffer->bufferSize = bufferData->bufferSize;
+            kernelDataBuffer->bufferDescriptor = aDescriptor;
+            
+            USBLog(3, "%s[%p]::LowLatencyPrepareBuffer  finished preparing data buffer: %p, size %d, desc: %p, cookie: %ld", getName(), this,
+                    kernelDataBuffer->bufferAddress, kernelDataBuffer->bufferSize, kernelDataBuffer->bufferDescriptor, kernelDataBuffer->cookie);
+        }
+        else if ( bufferData->bufferType == kUSBLowLatencyFrameListBuffer )
+        {
+            // We have a frame list that we need to map to the kernel's memory space
+            //
+            // Create a memory descriptor for our frame list and prepare it (pages it in if necesary and prepares it). 
+            //
+            aDescriptor = IOMemoryDescriptor::withAddress((vm_address_t)bufferData->bufferAddress, bufferData->bufferSize, kIODirectionOutIn, fTask);
+            if(!aDescriptor) 
+            {
+                USBLog(1,"%s[%p]::LowLatencyPrepareBuffer  Could not create a frame list memory descriptor (addr: %p, size %d)!", getName(), this, bufferData->bufferAddress, bufferData->bufferSize );
+                ret = kIOReturnNoMemory;
+                goto ErrorExit;
+            }
+            
+            ret = aDescriptor->prepare();
+            if (ret != kIOReturnSuccess)
+            {
+                USBLog(1,"%s[%p]::LowLatencyPrepareBuffer  Could not prepare the frame list memory descriptor!", getName(), this );
+                ret = kIOReturnNoMemory;
+                goto ErrorExit;
+            }
+            
+
+            // Map it into the kernel
+            //
+            frameListMap = aDescriptor->map();
+            if (!frameListMap) 
+            {
+                USBLog(1,"%s[%p]::LowLatencyPrepareBuffer  Could not prepare the frame list memory descriptor!", getName(), this );
+                ret = kIOReturnNoMemory;
+                aDescriptor->complete();
+                goto ErrorExit;
+            }
+
+            // Get the the mapped in virtual address and save it
+            //
+            kernelDataBuffer->frameListKernelAddress = frameListMap->getVirtualAddress();
+            
+            // Save the rest of the items
+            // 
+            kernelDataBuffer->bufferAddress = bufferData->bufferAddress;
+            kernelDataBuffer->bufferSize = bufferData->bufferSize;
+            kernelDataBuffer->frameListDescriptor = aDescriptor;
+            kernelDataBuffer->frameListMap = frameListMap;
+
+            USBLog(3, "%s[%p]::LowLatencyPrepareBuffer  finished preparing frame list buffer: %p, size %d, desc: %p, map %p, kernel address: %p, cookie: %ld", getName(), this,
+                    kernelDataBuffer->bufferAddress, kernelDataBuffer->bufferSize, kernelDataBuffer->bufferDescriptor, kernelDataBuffer->frameListMap,
+                    kernelDataBuffer->frameListKernelAddress,  kernelDataBuffer->cookie);
+        }
+    }
+    else
+        ret = kIOReturnNotAttached;
+
+ErrorExit:
+
+    if (ret)
+        USBLog(3, "%s[%p]::LowLatencyPrepareBuffer - returning err %x", getName(), this, ret);
+
+    DecrementOutstandingIO();
+    
+    return ret;
+}
+
+IOReturn 
+IOUSBInterfaceUserClient::LowLatencyReleaseBuffer(LowLatencyUserBufferInfo *dataBuffer)
+{
+    LowLatencyUserClientBufferInfo *	kernelDataBuffer	= NULL;
+    IOReturn				ret 			= kIOReturnSuccess;
+    bool				found 			= false;
+    
+    IncrementOutstandingIO();
+    
+    USBLog(3, "+%s[%p]::LowLatencyReleaseBuffer for cookie: %ld", getName(), this, dataBuffer->cookie);
+
+    if (fOwner && !isInactive())
+    {
+        // We need to find the LowLatencyUserBufferInfo structure that contains
+        // this buffer and then remove it from the list and free the structure
+        // and the memory that was allocated for it
+        //
+        kernelDataBuffer = FindBufferCookieInList( dataBuffer->cookie );
+        if ( kernelDataBuffer == NULL )
+        {
+            USBLog(3, "+%s[%p]::LowLatencyReleaseBuffer cookie: %ld, could not find buffer in list", getName(), this, dataBuffer->cookie);
+            ret = kIOReturnBadArgument;
+            goto ErrorExit;
+        }
+        
+        // Now, remove this bufferData from the list
+        //
+        found = RemoveDataBufferFromList( kernelDataBuffer );
+        if ( !found )
+        {
+            USBLog(3, "+%s[%p]::LowLatencyReleaseBuffer cookie: %ld, could not remove buffer (%p) from list", getName(), this, dataBuffer->cookie);
+            ret = kIOReturnBadArgument;
+            goto ErrorExit;
+        }
+        
+        // Now, need to complete/release/free the objects we allocated in our prepare
+        //
+        if ( kernelDataBuffer->frameListMap )
+            kernelDataBuffer->frameListMap->release();
+            
+        if ( kernelDataBuffer->frameListDescriptor )
+        {
+            kernelDataBuffer->frameListDescriptor->complete();
+            kernelDataBuffer->frameListDescriptor->release();
+        }
+        
+        if ( kernelDataBuffer->bufferDescriptor )
+        {
+            kernelDataBuffer->bufferDescriptor->complete();
+            kernelDataBuffer->bufferDescriptor->release();
+        }
+                
+        
+        // Finally, deallocate our kernelDataBuffer
+        //
+        IOFree(kernelDataBuffer, sizeof(LowLatencyUserClientBufferInfo));
+        
+    }
+    else
+        ret = kIOReturnNotAttached;
+
+ErrorExit:
+
+    if (ret)
+        USBLog(3, "%s[%p]::LowLatencyReleaseBuffer - returning err %x", getName(), this, ret);
+
+    DecrementOutstandingIO();
+    return ret;
+}
+
+void
+IOUSBInterfaceUserClient::AddDataBufferToList( LowLatencyUserClientBufferInfo * insertBuffer )
+{
+    LowLatencyUserClientBufferInfo *	buffer;
+    
+    // Traverse the list looking for last buffer and insert ours into it
+    //
+    if ( fUserClientBufferInfoListHead == NULL )
+    {
+        fUserClientBufferInfoListHead = insertBuffer;
+        return;
+    }
+    
+    buffer = fUserClientBufferInfoListHead;
+    
+    while ( buffer->nextBuffer != NULL )
+    {
+        buffer = buffer->nextBuffer;
+    }
+    
+    // When we get here, nextBuffer is pointing to NULL.  Our insert buffer
+    // already has nextBuffer = NULL, so we just insert it
+    //
+    buffer->nextBuffer = insertBuffer;
+}
+
+LowLatencyUserClientBufferInfo *	
+IOUSBInterfaceUserClient::FindBufferCookieInList( UInt32 cookie)
+{
+    LowLatencyUserClientBufferInfo *	buffer;
+    bool				foundIt = true;
+    
+    // Traverse the list looking for this buffer
+    //
+    if ( fUserClientBufferInfoListHead == NULL )
+    {
+        return NULL;
+    }
+    
+    buffer = fUserClientBufferInfoListHead;
+    
+    // Now, we need to see if our cookie is the same as one in the buffer list
+    //
+    while ( buffer->cookie != cookie )
+    {
+        buffer = buffer->nextBuffer;
+        if ( buffer == NULL )
+        {
+            foundIt = false;
+            break;
+        }
+    }
+    
+    if ( foundIt )
+        return buffer;
+    else
+        return false;
+}
+
+ bool			
+IOUSBInterfaceUserClient::RemoveDataBufferFromList( LowLatencyUserClientBufferInfo *removeBuffer)
+{
+    LowLatencyUserClientBufferInfo *	buffer;
+    LowLatencyUserClientBufferInfo *	previousBuffer;
+    
+    // If our head is NULL, then this buffer does not exist in our list
+    //
+    if ( fUserClientBufferInfoListHead == NULL )
+    {
+        return false;
+    }
+    
+    buffer = fUserClientBufferInfoListHead;
+    
+    // if our removeBuffer is the first one in the list, then just update the head and
+    // exit
+    //
+    if ( buffer == removeBuffer )
+    {
+        fUserClientBufferInfoListHead = buffer->nextBuffer;
+    }
+    else
+    {    
+        // Need to start previousBuffer pointing to our initial buffer, in case we match
+        // the first time
+        //
+        previousBuffer = buffer;
+        
+        while ( buffer->nextBuffer != removeBuffer )
+        {
+            previousBuffer = buffer;
+            buffer = previousBuffer->nextBuffer;
+        }
+        
+        // When we get here, buffer is pointing to the same buffer as removeBuffer
+        // and previous buffer is pointing to the previous element in the link list,
+        // so, update the link in previous to point to removeBuffer->nextBuffer;
+        //
+        buffer->nextBuffer = removeBuffer->nextBuffer;
+    }
+    
+    return true;
+}
+
+
 // ASYNC METHODS
 
 IOReturn 
@@ -1701,65 +2034,102 @@ IOUSBInterfaceUserClient::DoIsochPipeAsync(OSAsyncReference asyncRef, IOUSBIsocS
 
 
 IOReturn 
-IOUSBInterfaceUserClient::DoLowLatencyIsochPipeAsync(OSAsyncReference asyncRef, IOUSBLowLatencyIsocStruct *stuff, IODirection direction)
+IOUSBInterfaceUserClient::DoLowLatencyIsochPipeAsync(OSAsyncReference asyncRef, IOUSBLowLatencyIsocStruct *isocInfo, IODirection direction)
 {
-    IOReturn 			ret;
-    IOUSBPipe *			pipeObj;
-    IOUSBLowLatencyIsocCompletion		tap;
-    IOMemoryDescriptor *	dataMem = NULL;
-    IOMemoryDescriptor *	countMem = NULL;
-    int				frameLen = 0;	// In bytes
-    LowLatencyIsoAsyncPB * 	pb = NULL;
-
+    IOReturn 				ret;
+    IOUSBPipe *				pipeObj;
+    IOUSBLowLatencyIsocCompletion	tap;
+    IOMemoryDescriptor *		aDescriptor		= NULL;
+    IOUSBLowLatencyIsocFrame *		pFrameList 		= NULL;
+    IOUSBLowLatencyCommand *		command 		= NULL;
+    LowLatencyUserClientBufferInfo *	dataBuffer		= NULL;
+    LowLatencyUserClientBufferInfo *	frameListDataBuffer	= NULL;
+        
     USBLog(7, "+%s[%p]::DoLowLatencyIsochPipeAsync", getName(), this);
     retain();
     IncrementOutstandingIO();		// to make sure LowLatencyIsoReqComplete is still around
     
     if (fOwner && !isInactive())
     {
-	pipeObj = GetPipeObj(stuff->fPipe);
+	pipeObj = GetPipeObj(isocInfo->fPipe);
 	if(pipeObj)
 	{
-	    frameLen = stuff->fNumFrames * sizeof(IOUSBLowLatencyIsocFrame);
 	    do {
-		dataMem = IOMemoryDescriptor::withAddress((vm_address_t)stuff->fBuffer, stuff->fBufSize, direction, fTask);
-		if(!dataMem) 
+                // First, attempt to get a command for our transfer
+                //
+                command = (IOUSBLowLatencyCommand *) fFreeUSBLowLatencyCommandPool->getCommand(false);
+                
+                // If we couldn't get a command, increase the allocation and try again
+                //
+                if ( command == NULL )
+                {
+                    IncreaseCommandPool();
+                    
+                    command = (IOUSBLowLatencyCommand *) fFreeUSBLowLatencyCommandPool->getCommand(false);
+                    if ( command == NULL )
+                    {
+                        USBLog(3,"%s[%p]::DoLowLatencyIsochPipeAsync Could not get a IOUSBLowLatencyIsocCommand",getName(),this);
+                        ret = kIOReturnNoResources;
+                        break;
+                    }
+                }
+                
+                USBLog(7,"%s[%p]::DoLowLatencyIsochPipeAsync: dataBuffer cookie: %ld, offset: %ld, frameList cookie: %ld, offset : %ld", getName(),this, isocInfo->fDataBufferCookie, isocInfo->fDataBufferOffset, isocInfo->fFrameListBufferCookie, isocInfo->fFrameListBufferOffset );
+                
+                // Find the buffer corresponding to the data buffer cookie:
+                //
+                dataBuffer = FindBufferCookieInList(isocInfo->fDataBufferCookie);
+                
+                if ( dataBuffer == NULL )
 		{
 		    ret = kIOReturnNoMemory;
 		    break;
 		}
-		ret = dataMem->prepare();
-		if(ret != kIOReturnSuccess)
-		    break;
-	
-		countMem = IOMemoryDescriptor::withAddress((vm_address_t)stuff->fFrameCounts, frameLen, kIODirectionOutIn, fTask);
-		if(!countMem) 
+                
+                // Create a new IOMD that is a subrange of our data buffer memory descriptor, and prepare it
+                //
+                aDescriptor = IOMemoryDescriptor::withSubRange( dataBuffer->bufferDescriptor, isocInfo->fDataBufferOffset, isocInfo->fBufSize, direction );
+                if ( aDescriptor == NULL )
 		{
 		    ret = kIOReturnNoMemory;
 		    break;
 		}
-		// Copy in requested transfers, we'll copy out result in completion routine
-		pb = (LowLatencyIsoAsyncPB *)IOMalloc(sizeof(LowLatencyIsoAsyncPB) + frameLen);
-		if(!pb) 
-		{
-		    ret = kIOReturnNoMemory;
-		    break;
-		}
-	
-		bcopy(asyncRef, pb->fAsyncRef, sizeof(OSAsyncReference));
-		pb->frameLen = frameLen;
-		pb->frameBase = stuff->fFrameCounts;
-		pb->dataMem = dataMem;
-		pb->countMem = countMem;
-		countMem->readBytes(0, pb->frames, frameLen);
-		tap.target = this;
+
+                // Prepare this descriptor
+                //
+                ret = aDescriptor->prepare();
+                if (ret != kIOReturnSuccess)
+                {
+                    break;
+                }
+                
+                // Find the buffer corresponding to the frame list cookie:
+                //
+                frameListDataBuffer = FindBufferCookieInList(isocInfo->fFrameListBufferCookie);
+                
+                // Get our virtual address by looking at the buffer data and adding in the offset that was passed in
+                //
+                pFrameList = (IOUSBLowLatencyIsocFrame *) ( (UInt32) frameListDataBuffer->frameListKernelAddress + isocInfo->fFrameListBufferOffset);
+                
+                // Copy the data into our command buffer
+                //
+                command->SetAsyncReference( asyncRef );
+                command->SetFrameBase( (void *) ((UInt32) frameListDataBuffer->bufferAddress + isocInfo->fFrameListBufferOffset));
+                command->SetDataBuffer( aDescriptor );
+                
+                // Populate our completion routine
+                //
+                tap.target = this;
 		tap.action = &LowLatencyIsoReqComplete;
-		tap.parameter = pb;
-		if(direction == kIODirectionOut)
-		    ret = pipeObj->Write(dataMem, stuff->fStartFrame, stuff->fNumFrames, pb->frames, &tap, stuff->fUpdateFrequency);
+		tap.parameter = command;
+                
+		if ( direction == kIODirectionOut )
+		    ret = pipeObj->Write(aDescriptor, isocInfo->fStartFrame, isocInfo->fNumFrames, pFrameList, &tap, isocInfo->fUpdateFrequency);
 		else
-		    ret = pipeObj->Read(dataMem, stuff->fStartFrame, stuff->fNumFrames, pb->frames, &tap, stuff->fUpdateFrequency);
-	    } while (false);
+		    ret = pipeObj->Read(aDescriptor, isocInfo->fStartFrame, isocInfo->fNumFrames,pFrameList, &tap, isocInfo->fUpdateFrequency);
+	    
+            } while (false);
+            
 	    pipeObj->release();
 	}
 	else
@@ -1771,12 +2141,14 @@ IOUSBInterfaceUserClient::DoLowLatencyIsochPipeAsync(OSAsyncReference asyncRef,
     if(kIOReturnSuccess != ret) 
     {
 	USBLog(3, "%s[%p]::DoLowLatencyIsochPipeAsync err 0x%x", getName(), this, ret);
-	if(dataMem)
-	    dataMem->release();
-	if(countMem)
-	    countMem->release();
-	if(pb)
-	    IOFree(pb, sizeof(*pb) + frameLen);
+                
+        if ( aDescriptor )
+            aDescriptor->release();
+            
+        // return command
+        //
+        fFreeUSBLowLatencyCommandPool->returnCommand(command);
+        
 	DecrementOutstandingIO();
         release();
     }
@@ -1976,23 +2348,13 @@ IOUSBInterfaceUserClient::stop(IOService * provider)
     
     USBLog(7, "+%s[%p]::stop(%p)", getName(), this, provider);
 
-    if (GetOutstandingIO() > 0)
-	USBError(1, "%s[%p]::stop called with outstanding IO!!", getName(), this);
-	
-    if (fGate)
+    // If we have any kernelDataBuffer pointers, then release them now
+    //
+    if (fUserClientBufferInfoListHead != NULL)
     {
-	IOWorkLoop		*wl = getWorkLoop();
-	if (wl)
-	{
-	    wl->removeEventSource(fGate);
-	}
-	else
-	{
-	    USBError(1, "%s[%p]::stop - have gate, but no valid workloop!", getName(), this);
-	}
-	fGate->release();
-	fGate = NULL;
+        ReleasePreparedDescriptors();
     }
+
     super::stop(provider);
 
     USBLog(7, "-%s[%p]::stop(%p)", getName(), this, provider);
@@ -2002,21 +2364,33 @@ IOUSBInterfaceUserClient::stop(IOService * provider)
 void 
 IOUSBInterfaceUserClient::free()
 {
+    IOReturn ret;
+
+    // USBLog(7, "+%s[%p]::free", getName(), this);
     
+    if ( fFreeUSBLowLatencyCommandPool )
+    {
+       //  USBLog(5, "+%s[%p]::free releasing fFreeUSBLowLatencyCommandPool", getName(), this);
+        fFreeUSBLowLatencyCommandPool->release();
+    }
+
     if (fGate)
     {
-	IOWorkLoop		*wl = getWorkLoop();
-	if (wl)
-	{
-	    wl->removeEventSource(fGate);
-	}
-	else
-	{
-	    USBError(1, "%s[%p]::free - have gate, but no valid workloop!", getName(), this);
-	}
-	fGate->release();
-	fGate = NULL;
+        if (fWorkLoop)
+        {
+            ret = fWorkLoop->removeEventSource(fGate);
+            // USBLog(5, "+%s[%p]::free releasing fWorkLoop", getName(), this);
+            fWorkLoop->release();
+            fWorkLoop = NULL;
+        }
+        else
+        {
+            USBError(1, "%s[%p]::free - have gate, but no valid workloop (%p)!", getName(), this, fWorkLoop);
+        }
+        fGate->release();
+        fGate = NULL;
     }
+        
     super::free();
 }
 
@@ -2054,7 +2428,7 @@ IOUSBInterfaceUserClient::didTerminate( IOService * provider, IOOptionBits optio
     // in which case we can just close our provider and IOKit will take care of the rest. Otherwise, we need to 
     // hold on to the device and IOKit will terminate us when we close it later
     USBLog(3, "%s[%p]::didTerminate isInactive = %d, outstandingIO = %d", getName(), this, isInactive(), fOutstandingIO);
-    if ( GetOutstandingIO() == 0 )
+    if ( fOutstandingIO == 0 )
 	fOwner->close(this);
     else
 	fNeedToClose = true;
@@ -2153,4 +2527,86 @@ IOUSBInterfaceUserClient::GetGatedOutstandingIO(OSObject *target, void *param1,
     return kIOReturnSuccess;
 }
 
+void
+IOUSBInterfaceUserClient::IncreaseCommandPool(void)
+{
+    int i;
+    
+    USBLog(3,"%s[%p] Adding (%d) to Command Pool", getName(), this, kSizeToIncrementLowLatencyCommandPool);
 
+    for (i = 0; i < kSizeToIncrementLowLatencyCommandPool; i++)
+    {
+        IOUSBLowLatencyCommand *command = IOUSBLowLatencyCommand::NewCommand();
+        if (command)
+            fFreeUSBLowLatencyCommandPool->returnCommand(command);
+    }
+    
+    fCurrentSizeOfCommandPool += kSizeToIncrementLowLatencyCommandPool;
+
+}
+
+void
+IOUSBInterfaceUserClient::ReleasePreparedDescriptors(void)
+{
+    LowLatencyUserClientBufferInfo *	kernelDataBuffer;
+    LowLatencyUserClientBufferInfo *	nextBuffer;
+
+    // If we have any kernelDataBuffer pointers, then release them now
+    //
+    if (fUserClientBufferInfoListHead != NULL)
+    {
+        USBLog(5, "+%s[%p]::stop: fUserClientBufferInfoListHead NOT NULL (%p) ", getName(), this, fUserClientBufferInfoListHead);
+    
+        nextBuffer = fUserClientBufferInfoListHead;
+        kernelDataBuffer = fUserClientBufferInfoListHead;
+        
+        // Traverse the list and release memory
+        //
+        while ( nextBuffer != NULL )
+        {
+            nextBuffer = kernelDataBuffer->nextBuffer;
+            
+            // Now, need to complete/release/free the objects we allocated in our prepare
+            //
+            if ( kernelDataBuffer->frameListMap )
+                kernelDataBuffer->frameListMap->release();
+                
+            if ( kernelDataBuffer->frameListDescriptor )
+            {
+                kernelDataBuffer->frameListDescriptor->complete();
+                kernelDataBuffer->frameListDescriptor->release();
+            }
+
+            if ( kernelDataBuffer->bufferDescriptor )
+            {
+                // We call prepare on the bufferDescriptor, so we need to call complete on it 
+                //
+                kernelDataBuffer->bufferDescriptor->complete();
+                kernelDataBuffer->bufferDescriptor->release();
+            }
+            
+            // Finally, deallocate our kernelDataBuffer
+            //
+            IOFree(kernelDataBuffer, sizeof(LowLatencyUserClientBufferInfo));
+            
+            kernelDataBuffer = nextBuffer;
+        }
+        
+        fUserClientBufferInfoListHead = NULL;
+    }
+}
+
+IOUSBLowLatencyCommand *
+IOUSBLowLatencyCommand::NewCommand()
+{
+    IOUSBLowLatencyCommand *me = new IOUSBLowLatencyCommand;
+    
+    return me;
+
+}
+
+void  			
+IOUSBLowLatencyCommand::SetAsyncReference(OSAsyncReference  ref)
+{
+    bcopy(ref, fAsyncRef, sizeof(OSAsyncReference));
+}
